from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..permissions import permission_workflow_template_view

from .mixins.workflow_template_transition_trigger_mixins import WorkflowTemplateTransitionTriggerViewTestMixin


class WorkflowTemplateTransitionTriggerViewTestCase(
    WorkflowTemplateTransitionTriggerViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template(add_test_document_type=True)
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()
        self._create_test_workflow_template_transition()

    def test_workflow_template_transition_trigger_list_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_transition_event_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_trigger_list_view_with_access(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_event_list_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.conf import settings
from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.documents.permissions import permission_document_type_edit
from mayan.apps.events.classes import EventType, ModelEventType
from mayan.apps.views.generics import (
    AddRemoveView, ConfirmView, FormView, SingleObjectCreateView,
    SingleObjectDeleteView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms import (
    IndexTemplateEventTriggerRelationshipFormSet, IndexTemplateFilteredForm,
    IndexTemplateNodeForm
)
from ..icons import (
    icon_document_type_index_templates, icon_index,
    icon_index_instances_rebuild, icon_index_instances_reset,
    icon_index_template_create, icon_index_template_delete,
    icon_index_template_document_types, icon_index_template_edit,
    icon_index_template_event_triggers, icon_index_template_list,
    icon_index_template_node_create, icon_index_template_node_delete,
    icon_index_template_node_edit, icon_index_template_node_tree_view
)
from ..links import link_index_template_create
from ..literals import RELATIONSHIP_NO, RELATIONSHIP_YES
from ..models.index_template_models import (
    IndexTemplate, IndexTemplateEventTrigger, IndexTemplateNode
)
from ..permissions import (
    permission_index_template_create, permission_index_template_delete,
    permission_index_template_edit, permission_index_template_rebuild,
    permission_index_template_view
)
from ..tasks import task_index_template_rebuild


class DocumentTypeIndexTemplateAddRemoveView(AddRemoveView):
    main_object_method_add_name = 'index_template_add'
    main_object_method_remove_name = 'index_template_remove'
    main_object_permission = permission_document_type_edit
    main_object_model = DocumentType
    main_object_pk_url_kwarg = 'document_type_id'
    secondary_object_model = IndexTemplate
    secondary_object_permission = permission_index_template_edit
    list_available_title = _('Available indexes')
    list_added_title = _('Indexes linked')
    related_field = 'index_templates'
    view_icon = icon_document_type_index_templates

    def get_actions_extra_kwargs(self):
        return {'user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Documents of this type will appear in the indexes linked '
                'when these are updated. Events of the documents of this '
                'type will trigger updates in the linked indexes.'
            ),
            'title': _(
                'Indexes linked to document type: %s'
            ) % self.main_object
        }


class IndexTemplateListView(SingleObjectListView):
    model = IndexTemplate
    object_permission = permission_index_template_view
    view_icon = icon_index_template_list

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_main_link': link_index_template_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Indexes group document automatically into levels. '
                'Indexes are defined using template whose markers are '
                'replaced with direct properties of documents like label '
                'or description, or that of extended properties like '
                'metadata.'
            ),
            'no_results_title': _('There are no index templates.'),
            'title': _('Index templates')
        }


class IndexTemplateCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create index')}
    fields = ('label', 'slug', 'enabled')
    model = IndexTemplate
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_template_list'
    )
    view_icon = icon_index_template_create
    view_permission = permission_index_template_create

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class IndexTemplateDeleteView(SingleObjectDeleteView):
    model = IndexTemplate
    object_permission = permission_index_template_delete
    pk_url_kwarg = 'index_template_id'
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_template_list'
    )
    view_icon = icon_index_template_delete

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Delete the index: %s?') % self.object
        }


class IndexTemplateDocumentTypeAddRemoveView(AddRemoveView):
    main_object_method_add_name = 'document_types_add'
    main_object_method_remove_name = 'document_types_remove'
    main_object_permission = permission_index_template_edit
    main_object_model = IndexTemplate
    main_object_pk_url_kwarg = 'index_template_id'
    secondary_object_model = DocumentType
    secondary_object_permission = permission_document_type_edit
    list_available_title = _('Available document types')
    list_added_title = _('Document types linked')
    related_field = 'document_types'
    view_icon = icon_index_template_document_types

    def get_actions_extra_kwargs(self):
        return {'user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Only the documents of the types selected will be shown '
                'in the index when built. Only the events of the documents '
                'of the types select will trigger updates in the index.'
            ),
            'title': _(
                'Document types linked to index: %s'
            ) % self.main_object
        }


class IndexTemplateEditView(SingleObjectEditView):
    fields = ('label', 'slug', 'enabled')
    model = IndexTemplate
    object_permission = permission_index_template_edit
    pk_url_kwarg = 'index_template_id'
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_template_list'
    )
    view_icon = icon_index_template_edit

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Edit index: %s') % self.object
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class IndexTemplateEventTriggerListView(ExternalObjectViewMixin, FormView):
    external_object_class = IndexTemplate
    external_object_permission = permission_index_template_edit
    external_object_pk_url_kwarg = 'index_template_id'
    form_class = IndexTemplateEventTriggerRelationshipFormSet
    view_icon = icon_index_template_event_triggers

    def dispatch(self, *args, **kwargs):
        EventType.refresh()
        return super().dispatch(*args, **kwargs)

    def form_valid(self, form):
        try:
            for instance in form:
                if instance.cleaned_data['relationship'] == RELATIONSHIP_NO:
                    try:
                        index_template_event_trigger = self.external_object.event_triggers.get(
                            stored_event_type_id=instance.cleaned_data['stored_event_type_id']
                        )
                    except self.external_object.event_triggers.model.DoesNotExist:
                        """Ignore non existing entries."""
                    else:
                        index_template_event_trigger._event_actor = self.request.user
                        index_template_event_trigger.delete()
                elif instance.cleaned_data['relationship'] == RELATIONSHIP_YES:
                    if not self.external_object.event_triggers.filter(
                        stored_event_type_id=instance.cleaned_data['stored_event_type_id']
                    ).exists():
                        index_template_event_trigger = IndexTemplateEventTrigger(
                            index_template=self.external_object,
                            stored_event_type_id=instance.cleaned_data['stored_event_type_id']
                        )
                        index_template_event_trigger._event_actor = self.request.user
                        index_template_event_trigger.save()
        except Exception as exception:
            messages.error(
                message=_(
                    'Error updating index template event trigger; %s'
                ) % exception, request=self.request

            )
            if settings.DEBUG or settings.TESTING:
                raise
        else:
            messages.success(
                message=_(
                    'Index template event triggers updated successfully.'
                ), request=self.request
            )

        return super().form_valid(form=form)

    def get_extra_context(self):
        return {
            'form_display_mode_table': True,
            'subtitle': _(
                'Triggers are document events that cause instances of this '
                'index template to be updated.'
            ),
            'object': self.external_object,
            'title': _(
                'Index template event triggers for: %s'
            ) % self.external_object
        }

    def get_initial(self):
        initial = []

        queryset = IndexTemplateEventTrigger.objects.filter(
            index_template=self.external_object
        )

        for stored_event_type in self.get_stored_event_types():
            if queryset.filter(stored_event_type=stored_event_type).exists():
                relationship = RELATIONSHIP_YES
            else:
                relationship = RELATIONSHIP_NO

            initial.append(
                {
                    'label': stored_event_type.label,
                    'namespace': stored_event_type.namespace,
                    'relationship': relationship,
                    'stored_event_type_id': stored_event_type.id
                }
            )
        return initial

    def get_stored_event_types(self):
        for event_type in ModelEventType.get_for_class(klass=Document):
            yield event_type.get_stored_event_type()


class IndexTemplateNodeListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_class = IndexTemplate
    external_object_permission = permission_index_template_edit
    external_object_pk_url_kwarg = 'index_template_id'
    view_icon = icon_index_template_node_tree_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.external_object,
            'title': _(
                'Tree template nodes for index: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.index_template_root_node.get_descendants(
            include_self=True
        )


class IndexTemplateNodeCreateView(SingleObjectCreateView):
    form_class = IndexTemplateNodeForm
    model = IndexTemplateNode
    view_icon = icon_index_template_node_create

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_parent_node().index, permissions=(
                permission_index_template_edit,
            ), user=request.user
        )

        return super().dispatch(request, *args, **kwargs)

    def get_extra_context(self):
        return {
            'object': self.get_parent_node().index,
            'title': _('Create child node of: %s') % self.get_parent_node()
        }

    def get_initial(self):
        parent_node = self.get_parent_node()
        return {
            'index': parent_node.index, 'parent': parent_node
        }

    def get_parent_node(self):
        return get_object_or_404(
            klass=IndexTemplateNode, pk=self.kwargs['index_template_node_id']
        )


class IndexTemplateNodeDeleteView(SingleObjectDeleteView):
    model = IndexTemplateNode
    object_permission = permission_index_template_edit
    pk_url_kwarg = 'index_template_node_id'
    view_icon = icon_index_template_node_delete

    def get_extra_context(self):
        return {
            'index': self.object.index,
            'navigation_object_list': ('index', 'node'),
            'node': self.object,
            'title': _(
                'Delete the index template node: %s?'
            ) % self.object
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='indexing:index_template_view', kwargs={
                'index_template_id': self.object.index.pk
            }
        )


class IndexTemplateNodeEditView(SingleObjectEditView):
    form_class = IndexTemplateNodeForm
    model = IndexTemplateNode
    object_permission = permission_index_template_edit
    pk_url_kwarg = 'index_template_node_id'
    view_icon = icon_index_template_node_edit

    def get_extra_context(self):
        return {
            'index': self.object.index,
            'navigation_object_list': ('index', 'node'),
            'node': self.object,
            'title': _(
                'Edit the index template node: %s'
            ) % self.object
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='indexing:index_template_view', kwargs={
                'index_template_id': self.object.index.pk
            }
        )


class IndexTemplateRebuildView(ConfirmView):
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_template_list'
    )
    view_icon = icon_index_instances_rebuild

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Rebuild index template: %s') % self.get_object()
        }

    def get_object(self):
        return get_object_or_404(
            klass=self.get_queryset(), pk=self.kwargs['index_template_id']
        )

    def get_queryset(self):
        return AccessControlList.objects.restrict_queryset(
            permission=permission_index_template_rebuild,
            queryset=IndexTemplate.objects.all(), user=self.request.user
        )

    def view_action(self):
        task_index_template_rebuild.apply_async(
            kwargs={
                'index_id': self.get_object().pk
            }
        )

        messages.success(
            message=_('Index template queued for rebuild.'),
            request=self.request
        )


class IndexTemplateAllRebuildView(FormView):
    extra_context = {
        'title': _('Rebuild index templates'),
    }
    form_class = IndexTemplateFilteredForm
    view_icon = icon_index_instances_rebuild

    def form_valid(self, form):
        count = 0
        for index in form.cleaned_data['index_templates']:
            task_index_template_rebuild.apply_async(
                kwargs={
                    'index_id': index.pk
                }
            )
            count += 1

        messages.success(
            message=ungettext(
                singular='%(count)d index template queued for rebuild.',
                plural='%(count)d index templates queued for rebuild.',
                number=count
            ) % {
                'count': count,
            }, request=self.request
        )

        return super().form_valid(form=form)

    def get_form_extra_kwargs(self):
        return {
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')


class IndexTemplateResetView(FormView):
    extra_context = {
        'title': _('Reset indexes'),
    }
    form_class = IndexTemplateFilteredForm
    view_icon = icon_index_instances_reset

    def form_valid(self, form):
        count = 0
        for index in form.cleaned_data['index_templates']:
            index.reset()
            count += 1

        messages.success(
            message=ungettext(
                singular='%(count)d index reset.',
                plural='%(count)d indexes reset.',
                number=count
            ) % {
                'count': count,
            }, request=self.request
        )

        return super().form_valid(form=form)

    def get_form_extra_kwargs(self):
        return {
            'help_text': _(
                'Index templates for which their instances will be deleted.'
            ),
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')



import logging

from mayan.apps.document_indexing.tasks import task_index_instance_document_add

logger = logging.getLogger(name=__name__)


def handler_index_document(sender, **kwargs):
    if not kwargs.get('created', False):
        for document in kwargs['instance'].documents.all():
            task_index_instance_document_add.apply_async(
                kwargs={'document_id': document.pk}
            )


def handler_cabinet_pre_delete(sender, **kwargs):
    for document in kwargs['instance'].documents.all():
        # Remove each of the related documents.
        # Trigger the remove event for each document so they can be
        # reindexed.
        kwargs['instance']._document_remove(document=document)



import logging

from django.apps import apps
from django.db import OperationalError

from mayan.celery import app

logger = logging.getLogger(name=__name__)


@app.task(ignore_result=True)
def task_download_files_stale_delete():
    logger.debug(msg='Executing')

    DownloadFile = apps.get_model(
        app_label='storage', model_name='DownloadFile'
    )

    logger.debug('Start')

    DownloadFile.objects.stale_delete()

    logger.debug('Finished')


@app.task(ignore_result=True)
def task_shared_upload_stale_delete():
    logger.debug('Executing')

    SharedUploadedFile = apps.get_model(
        app_label='storage', model_name='SharedUploadedFile'
    )

    logger.debug('Start')

    SharedUploadedFile.objects.stale_delete()

    logger.debug('Finished')


@app.task(bind=True, ignore_result=True, retry_backoff=True)
def task_shared_upload_delete(self, shared_uploaded_file_id):
    SharedUploadedFile = apps.get_model(
        app_label='storage', model_name='SharedUploadedFile'
    )

    shared_uploaded_file = SharedUploadedFile.objects.get(
        pk=shared_uploaded_file_id
    )

    try:
        shared_uploaded_file.delete()
    except OperationalError as exception:
        logger.warning(
            'Operational error attempting to delete shared upload file: '
            '%s; %s. Retrying.', shared_uploaded_file, exception
        )
        raise self.retry(exc=exception)



from furl import furl

from django.conf import settings
from django.test import override_settings
from django.urls import reverse
from django.utils.http import urlunquote_plus

from mayan.apps.smart_settings.classes import SettingNamespace
from mayan.apps.testing.tests.base import GenericViewTestCase
from mayan.apps.user_management.events import event_user_edited

from ..classes import AuthenticationBackend
from ..events import event_user_logged_in, event_user_logged_out

from .literals import (
    PATH_AUTHENTICATION_BACKEND_EMAIL, PATH_AUTHENTICATION_BACKEND_USERNAME,
)
from .mixins import LoginViewTestMixin, LogoutViewTestMixin


class LoginTestCase(LoginViewTestMixin, GenericViewTestCase):
    """
    Test that users can login via the supported authentication methods.
    """
    authenticated_url = reverse(viewname='common:home')
    authentication_url = urlunquote_plus(
        furl(
            path=reverse(settings.LOGIN_URL), args={
                'next': authenticated_url
            }
        ).tostr()
    )
    auto_login_user = False
    create_test_case_super_user = True

    def setUp(self):
        super().setUp()
        SettingNamespace.invalidate_cache_all()

    def test_non_authenticated_request(self):
        self._clear_events()

        response = self._request_authenticated_view()
        self.assertRedirects(
            response=response, expected_url=self.authentication_url
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL)
    def test_login_view_with_email(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_email()
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL)
    def test_login_view_with_email_and_dont_remember_me(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_email(
            extra_data={'remember_me': False}
        )
        self.assertEqual(response.status_code, 302)
        self.assertTrue(self.client.session.get_expire_at_browser_close())

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_EMAIL)
    def test_login_view_with_email_and_remember_me(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_email(
            extra_data={'remember_me': True}
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            self.client.session.get_expiry_age(),
            AuthenticationBackend.cls_get_instance().maximum_session_length
        )
        self.assertFalse(self.client.session.get_expire_at_browser_close())

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def test_login_view_with_username(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_username()
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def test_login_view_with_username_and_dont_remember_me(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_username(
            extra_data={'remember_me': False}
        )
        self.assertEqual(response.status_code, 302)
        self.assertTrue(self.client.session.get_expire_at_browser_close())

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def test_login_view_with_username_and_remember_me(self):
        AuthenticationBackend.cls_initialize()

        self._clear_events()

        response = self._request_login_view_with_username(
            extra_data={'remember_me': True}
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            self.client.session.get_expiry_age(),
            AuthenticationBackend.cls_get_instance().maximum_session_length
        )
        self.assertFalse(self.client.session.get_expire_at_browser_close())

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)

    @override_settings(AUTHENTICATION_BACKEND=PATH_AUTHENTICATION_BACKEND_USERNAME)
    def test_login_view_redirect_with_username(self):
        AuthenticationBackend.cls_initialize()

        TEST_REDIRECT_URL = reverse(viewname='common:about_view')

        self._clear_events()

        response = self._request_login_view_with_username(
            query={'next': TEST_REDIRECT_URL}, follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(TEST_REDIRECT_URL, 302)])

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_user_edited.id)

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_super_user)
        self.assertEqual(events[1].target, self._test_case_super_user)
        self.assertEqual(events[1].verb, event_user_logged_in.id)


class LogoutViewTestCase(LogoutViewTestMixin, GenericViewTestCase):
    def test_logout_view(self):
        self._clear_events()

        request = self._request_logout_view()
        self.assertEqual(request.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_case_user)
        self.assertEqual(events[0].verb, event_user_logged_out.id)



from mayan.apps.testing.tests.base import BaseTestCase

from ..search_query_terms import QueryTerm


class SearchTermDecodeEmptyTestCase(BaseTestCase):
    def test_term_decode_single(self):
        query = {'q': ''}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 0)

    def test_term_decode_empty_quoted_term(self):
        query = {'q': '""'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, '')

    def test_term_decode_empty_quoted_term_double(self):
        query = {'q': '"" ""'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 2)

        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, '')

        self.assertEqual(query_term[1].is_meta, False)
        self.assertEqual(query_term[1].is_quoted, True)
        self.assertEqual(query_term[1].is_raw, False)
        self.assertEqual(query_term[1].text, '')


class SearchTermDecodeTestCase(BaseTestCase):
    def test_term_decode_single(self):
        query = {'q': 'term1'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, 'term1')

    def test_term_decode_dual(self):
        query = {'q': 'term1 term2'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 2)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, 'term1')

        self.assertEqual(query_term[1].is_meta, False)
        self.assertEqual(query_term[1].is_quoted, False)
        self.assertEqual(query_term[1].is_raw, False)
        self.assertEqual(query_term[1].text, 'term2')

    def test_term_decode_dual_with_operator(self):
        query = {'q': 'term1 AND term2'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 3)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, 'term1')

        self.assertEqual(query_term[1].is_meta, True)
        self.assertEqual(query_term[1].is_quoted, False)
        self.assertEqual(query_term[1].is_raw, False)
        self.assertEqual(query_term[1].text, 'AND')

        self.assertEqual(query_term[2].is_meta, False)
        self.assertEqual(query_term[2].is_quoted, False)
        self.assertEqual(query_term[2].is_raw, False)
        self.assertEqual(query_term[2].text, 'term2')


class SearchInterpreterTermDecodeQuotedTestCase(BaseTestCase):
    def test_term_decode_single_literal(self):
        query = {'q': '"term1 AND term2"'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, 'term1 AND term2')

    def test_term_decode_dual_literal_with_operator(self):
        query = {'q': '"term1" AND "term2 term3"'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 3)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, 'term1')

        self.assertEqual(query_term[1].is_meta, True)
        self.assertEqual(query_term[1].is_quoted, False)
        self.assertEqual(query_term[1].is_raw, False)
        self.assertEqual(query_term[1].text, 'AND')

        self.assertEqual(query_term[2].is_meta, False)
        self.assertEqual(query_term[2].is_quoted, True)
        self.assertEqual(query_term[2].is_raw, False)
        self.assertEqual(query_term[2].text, 'term2 term3')


class SearchTermDecodeRawTestCase(BaseTestCase):
    def test_term_decode_raw(self):
        query = {'q': '`term1`'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, True)
        self.assertEqual(query_term[0].text, 'term1')

    def test_term_decode_raw_multiple_words(self):
        query = {'q': '`term1 term2`'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, True)
        self.assertEqual(query_term[0].text, 'term1 term2')

    def test_term_decode_quoted_raw(self):
        query = {'q': '"`term1`"'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, '`term1`')

    def test_term_decode_quoted_multiple_words_raw(self):
        query = {'q': '"`term1 term2`"'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, True)
        self.assertEqual(query_term[0].is_raw, False)
        self.assertEqual(query_term[0].text, '`term1 term2`')

    def test_term_decode_raw_quoted_multiple_words(self):
        query = {'q': '`"term1 term2"`'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 1)
        self.assertEqual(query_term[0].is_meta, False)
        self.assertEqual(query_term[0].is_quoted, False)
        self.assertEqual(query_term[0].is_raw, True)
        self.assertEqual(query_term[0].text, '"term1 term2"')

    def test_term_decode_quoted_unterminated(self):
        query = {'q': '"term1'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 0)

    def test_term_decode_raw_unterminated(self):
        query = {'q': '`term1'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 0)

    def test_term_decode_raw_unterminated_quote(self):
        query = {'q': '`term1"'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 0)

    def test_term_decode_raw_unterminated_quote_double_terms(self):
        query = {'q': '`term1 "term2'}

        query_terms = QueryTerm.do_query_parse(query=query)
        query_term = query_terms['q']

        self.assertEqual(len(query_term), 0)



from collections import OrderedDict

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import REDIRECT_FIELD_NAME, login as django_auth_login
from django.contrib.auth.forms import SetPasswordForm
from django.contrib.auth.views import (
    LoginView, LogoutView, PasswordChangeDoneView, PasswordChangeView,
    PasswordResetCompleteView, PasswordResetConfirmView,
    PasswordResetDoneView, PasswordResetView,
    SuccessURLAllowedHostsMixin
)
from django.core.exceptions import PermissionDenied
from django.forms import formsets
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, resolve_url
from django.urls import reverse, reverse_lazy
from django.utils.decorators import classonlymethod, method_decorator
from django.utils.translation import ungettext, ugettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.debug import sensitive_post_parameters

from formtools.wizard.views import SessionWizardView, StepsHelper
from stronghold.decorators import public
from stronghold.views import StrongholdPublicMixin

import mayan
from mayan.apps.common.settings import setting_home_view
from mayan.apps.organizations.utils import get_organization_installation_url
from mayan.apps.user_management.permissions import permission_user_edit
from mayan.apps.user_management.querysets import (
    get_all_users_queryset, get_user_queryset
)
from mayan.apps.views.generics import MultipleObjectFormActionView
from mayan.apps.views.view_mixins import ViewIconMixin
from mayan.apps.views.http import URL

from ..classes import AuthenticationBackend
from ..forms import AuthenticationFormBase
from ..icons import icon_password_change
from ..literals import SESSION_MULTI_FACTOR_USER_ID_KEY
from ..settings import setting_disable_password_reset


class MultiFactorAuthenticationView(
    SuccessURLAllowedHostsMixin, SessionWizardView
):
    redirect_field_name = REDIRECT_FIELD_NAME
    template_name = 'authentication/login.html'

    # Login view methods.
    get_success_url = LoginView.get_success_url
    get_redirect_url = LoginView.get_redirect_url

    @staticmethod
    def condition_dict(self):
        self.authentication_backend = AuthenticationBackend.cls_get_instance()
        condition_dict = self.authentication_backend.get_condition_dict()
        return condition_dict

    @staticmethod
    def form_list_property(self):
        """
        Return the processed form list after the view has initialized.
        """
        self.authentication_backend = AuthenticationBackend.cls_get_instance()
        form_list = self.authentication_backend.get_form_list()
        computed_form_list = OrderedDict()

        for form_index, form in enumerate(iterable=form_list):
            computed_form_list[str(form_index)] = form

        for form in computed_form_list.values():
            if issubclass(form, formsets.BaseFormSet):
                form = form.form

        return computed_form_list

    @classonlymethod
    def as_view(cls, *args, **kwargs):
        # SessionWizardView needs at least one form in order to be
        # initialized as a view. Declare one empty form and then change the
        # form list in the .dispatch() method.
        class EmptyForm(AuthenticationFormBase):
            """Empty form"""

        cls.form_list = [EmptyForm]

        # Allow super to initialize and pass the form_list len() assert
        # before replacing the form_list attribute with our property.
        result = super().as_view(*args, **kwargs)

        def null_setter(self, value):
            return

        cls.form_list = property(
            fget=MultiFactorAuthenticationView.form_list_property,
            fset=null_setter
        )
        cls.condition_dict = property(
            fget=MultiFactorAuthenticationView.condition_dict,
            fset=null_setter
        )

        return result

    @method_decorator(sensitive_post_parameters())
    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    @method_decorator(public)
    def dispatch(self, request, *args, **kwargs):
        steps = StepsHelper(self)

        if steps.count == 0:
            user_id = request.session.get(
                SESSION_MULTI_FACTOR_USER_ID_KEY, None
            )
            if user_id:
                return self.done()
            else:
                return HttpResponseRedirect(
                    redirect_to=resolve_url(
                        to=settings.LOGIN_URL
                    )
                )
        else:
            return super().dispatch(request, *args, **kwargs)

    def done(self, form_list=None, **kwargs):
        """
        Perform the same function as Django's LoginView.form_valid().
        """
        kwargs = self.get_all_cleaned_data()
        self.authentication_backend.process(
            form_list=form_list, kwargs=kwargs, request=self.request
        )
        user = self.authentication_backend.get_user(
            form_list=form_list, kwargs=kwargs, request=self.request
        )

        django_auth_login(request=self.request, user=user)

        return HttpResponseRedirect(
            redirect_to=self.get_success_url()
        )

    def get_context_data(self, form, **kwargs):
        context = super().get_context_data(form=form, **kwargs)

        context.update(
            AuthenticationBackend.cls_get_instance().get_context_data()
        )

        wizard_step = self.form_list[self.steps.current]

        if self.storage.current_step == self.steps.last:
            context['submit_label'] = None
        else:
            context['submit_label'] = _('Next')

        context.update(
            {
                'form_css_classes': 'form-hotkey-double-click',
                'step_title': _(
                    'Step %(step)d of %(total_steps)d: %(step_label)s'
                ) % {
                    'step': self.steps.step1,
                    'step_label': wizard_step._label,
                    'total_steps': len(self.form_list)
                },
                'wizard_step': wizard_step,
                'wizard_steps': self.form_list
            }
        )
        return context

    def get_form_kwargs(self, step):
        kwargs = super().get_form_kwargs()
        kwargs['request'] = self.request
        kwargs['wizard'] = self
        return kwargs


class MayanLoginView(StrongholdPublicMixin, LoginView):
    extra_context = {
        'appearance_type': 'plain'
    }
    redirect_authenticated_user = True
    template_name = 'authentication/login.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            AuthenticationBackend.cls_get_instance().get_context_data()
        )
        return context

    def get_form_class(self):
        return AuthenticationBackend.cls_get_instance().get_login_form_class()

    def form_valid(self, form):
        if not AuthenticationBackend.cls_get_instance().form_list:
            AuthenticationBackend.cls_get_instance().process(
                kwargs=form.cleaned_data, request=self.request

            )
            return super().form_valid(form=form)
        else:
            self.request.session[
                SESSION_MULTI_FACTOR_USER_ID_KEY
            ] = form.get_user().pk

            url = URL(
                path=reverse(
                    viewname='authentication:multi_factor_authentication_view'
                )
            )

            next_url = self.get_redirect_url()
            if next_url:
                url.args[self.redirect_field_name] = next_url

            return HttpResponseRedirect(
                redirect_to=url.to_string()
            )


class MayanLogoutView(LogoutView):
    """No current change or overrides, left here for future expansion"""


class MayanPasswordChangeDoneView(PasswordChangeDoneView):
    def dispatch(self, *args, **kwargs):
        messages.success(
            message=_('Your password has been successfully changed.'),
            request=self.request
        )
        return redirect(
            to=self.request.user.get_absolute_url()
        )


class MayanPasswordChangeView(ViewIconMixin, PasswordChangeView):
    extra_context = {
        'title': _('Current user password change')
    }
    success_url = reverse_lazy(
        viewname='authentication:password_change_done'
    )
    template_name = 'appearance/generic_form.html'
    view_icon = icon_password_change

    def dispatch(self, *args, **kwargs):
        if self.request.user.user_options.block_password_change:
            messages.error(
                message=_(
                    'Changing the password is not allowed for this account.'
                ), request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(viewname=setting_home_view.view)
            )

        return super().dispatch(*args, **kwargs)


class MayanPasswordResetCompleteView(
    StrongholdPublicMixin, PasswordResetCompleteView
):
    extra_context = {
        'appearance_type': 'plain'
    }
    template_name = 'authentication/password_reset_complete.html'


class MayanPasswordResetConfirmView(
    StrongholdPublicMixin, PasswordResetConfirmView
):
    extra_context = {
        'appearance_type': 'plain'
    }
    success_url = reverse_lazy(
        viewname='authentication:password_reset_complete_view'
    )
    template_name = 'authentication/password_reset_confirm.html'


class MayanPasswordResetDoneView(StrongholdPublicMixin, PasswordResetDoneView):
    extra_context = {
        'appearance_type': 'plain'
    }
    template_name = 'authentication/password_reset_done.html'


class MayanPasswordResetView(StrongholdPublicMixin, PasswordResetView):
    email_template_name = 'authentication/password_reset_email.html'
    extra_context = {
        'appearance_type': 'plain'
    }
    subject_template_name = 'authentication/password_reset_subject.txt'
    success_url = reverse_lazy(
        viewname='authentication:password_reset_done_view'
    )
    template_name = 'authentication/password_reset_form.html'

    # Hardcoded overloaded method to allow adding extra email context from
    # a method and not just the Django provided `self.extra_email_context`.
    # On each new Django version, verify if this method has changed and
    # update this overloading.
    def form_valid(self, form):
        opts = {
            'email_template_name': self.email_template_name,
            'extra_email_context': self.get_extra_email_context(),
            'from_email': self.from_email,
            'html_email_template_name': self.html_email_template_name,
            'request': self.request,
            'subject_template_name': self.subject_template_name,
            'token_generator': self.token_generator,
            'use_https': self.request.is_secure()
        }
        form.save(**opts)
        # Specify the super class `PasswordResetView` explicitly to avoid
        # executing `form_valid` again.
        return super(PasswordResetView, self).form_valid(form=form)

    def get(self, *args, **kwargs):
        if setting_disable_password_reset.value:
            return redirect(to=setting_home_view.value)
        return super().get(*args, **kwargs)

    def get_extra_email_context(self):
        extra_email_context_project_website = get_organization_installation_url(
            request=self.request
        )
        extra_email_context = {
            'project_copyright': mayan.__copyright__,
            'project_license': mayan.__license__,
            'project_title': mayan.__title__,
            'project_website': str(extra_email_context_project_website)
        }
        return extra_email_context

    def post(self, *args, **kwargs):
        if setting_disable_password_reset.value:
            return redirect(to=setting_home_view.value)
        return super().post(*args, **kwargs)


class UserSetPasswordView(MultipleObjectFormActionView):
    form_class = SetPasswordForm
    object_permission = permission_user_edit
    pk_url_kwarg = 'user_id'
    source_queryset = get_user_queryset()
    success_message = _(
        'Password change request performed on %(count)d user'
    )
    success_message_plural = _(
        'Password change request performed on %(count)d users'
    )
    view_icon = icon_password_change

    def dispatch(self, request, *args, **kwargs):
        result = super().dispatch(request=request, *args, **kwargs)

        queryset = self.get_queryset(
            source_queryset=get_all_users_queryset()
        )

        queryset_staff_users = queryset.filter(is_staff=True)
        queryset_super_users = queryset.filter(is_superuser=True)

        if queryset_staff_users.exists() or queryset_super_users.exists():
            messages.warning(
                message=_(
                    'Changing the password of staff or super user '
                    'accounts via the user interface is not allowed. '
                    'Use administration tools to perform this '
                    'operation.'
                ), request=self.request
            )

        return result

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Change user password',
                plural='Change users passwords',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Change password for user: %s'
                    ) % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        queryset = self.object_list
        result = {}
        if queryset:
            result['user'] = queryset.first()
            return result
        else:
            raise PermissionDenied

    def object_action(self, form, instance):
        try:
            instance.set_password(
                raw_password=form.cleaned_data['new_password1']
            )
            instance.save()
            messages.success(
                message=_(
                    'Successful password reset for user: %s.'
                ) % instance, request=self.request
            )
        except Exception as exception:
            messages.error(
                message=_(
                    'Error reseting password for user "%(user)s": %(error)s'
                ) % {
                    'error': exception, 'user': instance
                }, request=self.request
            )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Tags'), name='tags'
)

event_tag_attached = namespace.add_event_type(
    label=_('Tag attached to document'), name='attach'
)
event_tag_created = namespace.add_event_type(
    label=_('Tag created'), name='tag_created'
)
event_tag_edited = namespace.add_event_type(
    label=_('Tag edited'), name='tag_edited'
)
event_tag_removed = namespace.add_event_type(
    label=_('Tag removed from document'), name='remove'
)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('messaging', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='message',
            name='parent',
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.user_management.querysets import get_user_queryset
from mayan.apps.views.widgets import TextAreaDiv

from .models import Message


class MessageCreateForm(forms.ModelForm):
    class Meta:
        fields = ('user', 'subject', 'body')
        model = Message

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['body'].widget.attrs = {
            'class': 'full-height', 'data-height-difference': 560
        }
        self.fields['user'].queryset = get_user_queryset()
        self.fields['user'].widget.attrs = {'class': 'select2'}


class MessageDetailForm(forms.Form):
    body = forms.CharField(
        label=_('Body'),
        widget=TextAreaDiv(
            attrs={
                'class': 'views-text-wrap full-height',
                'data-height-difference': 360
            }
        )
    )

    def __init__(self, *args, **kwargs):
        self.instance = kwargs.pop('instance', None)
        super().__init__(*args, **kwargs)
        self.fields['body'].initial = self.instance.body



import json

from django.db.models import Q

from ...models.workflow_state_action_models import WorkflowStateAction

from ..literals import (
    DOCUMENT_WORKFLOW_LAUNCH_ACTION_CLASS_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_DOTTED_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_GENERIC_DOTTED_PATH,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
    TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN
)

from .workflow_instance_mixins import WorkflowInstanceTestMixin
from .workflow_template_state_mixins import WorkflowTemplateStateTestMixin


class WorkflowTemplateStateActionTestMixin(
    WorkflowInstanceTestMixin, WorkflowTemplateStateTestMixin
):
    _test_workflow_template_state_action_path = TEST_WORKFLOW_TEMPLATE_STATE_ACTION_GENERIC_DOTTED_PATH
    auto_add_test_workflow_template_test_document_type = True
    auto_create_test_workflow_template = True
    auto_create_test_workflow_template_state = True
    auto_create_test_document_stub = False
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._test_workflow_template_state_actions = []
        self._create_test_document_stub()

    def _create_test_workflow_template_state_action(
        self, extra_backend_data=None, extra_data=None,
        workflow_state_index=0
    ):
        total_test_workflow_template_state_actions = len(
            self._test_workflow_template_state_actions
        )
        label = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            total_test_workflow_template_state_actions
        )

        backend_data = {}

        if extra_backend_data:
            backend_data.update(**extra_backend_data)

        data = {
            'backend_data': json.dumps(obj=backend_data),
            'backend_path': self._test_workflow_template_state_action_path,
            'label': label
        }

        if extra_data:
            data.update(extra_data)

        self._test_workflow_template_state_action = self._test_workflow_template_states[
            workflow_state_index
        ].actions.create(**data)

        self._test_workflow_template_state_actions.append(
            self._test_workflow_template_state_action
        )

    def _execute_workflow_template_state_action(
        self, klass, context=None, kwargs=None
    ):
        final_context = {
            'workflow_instance': self._test_workflow_instance
        }

        if context:
            final_context.update(context)

        final_kwargs = {
            'model_instance_id': 1  # Fake ID to allow direct instance.
        }

        if kwargs:
            final_kwargs.update(kwargs)

        action = klass(**final_kwargs)

        return action.execute(context=final_context)


class WorkflowTemplateStateActionAPIViewTestMixin(
    WorkflowTemplateStateActionTestMixin
):
    def _request_test_workflow_template_state_action_create_api_view(self):
        pk_list = list(
            WorkflowStateAction.objects.values_list('pk', flat=True)
        )

        total_test_workflow_template_state_actions = len(
            self._test_workflow_template_state_actions
        )
        label = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            total_test_workflow_template_state_actions
        )

        data = {
            'label': label,
            'backend_path': self._test_workflow_template_state_action_path
        }

        response = self.post(
            viewname='rest_api:workflow-template-state-action-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data=data
        )

        try:
            self._test_workflow_template_state_action = WorkflowStateAction.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowStateAction.DoesNotExist:
            self._test_workflow_template_state_action = None

        return response

    def _request_test_workflow_template_state_action_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-action-list', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_action_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_state_action_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:workflow-template-state-action-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
                'backend_path': self._test_workflow_template_state_action_path
            }
        )


class WorkflowTemplateStateActionLaunchViewTestMixin(
    WorkflowTemplateStateActionTestMixin
):
    def _request_document_workflow_template_launch_action_create_view(
        self, extra_data=None
    ):
        data = {
            'enabled': True,
            'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            'when': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN
        }

        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'backend_path': DOCUMENT_WORKFLOW_LAUNCH_ACTION_CLASS_PATH
            }, data=data
        )


class WorkflowTemplateStateActionViewTestMixin(
    WorkflowTemplateStateActionTestMixin
):
    def _request_test_workflow_template_state_action_create_get_view(
        self, backend_path
    ):
        return self.get(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'backend_path': backend_path

            }
        )

    def _request_test_workflow_template_state_action_create_post_view(
        self, backend_path, extra_data=None
    ):
        data = {
            'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL,
            'when': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_WHEN
        }
        if extra_data:
            data.update(extra_data)

        pk_list = list(
            WorkflowStateAction.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk,
                'backend_path': backend_path
            }, data=data
        )

        try:
            self._test_workflow_template_state_action = WorkflowStateAction.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowStateAction.DoesNotExist:
            self._test_workflow_template_state_action = None

        return response

    def _request_test_workflow_template_state_action_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_delete',
            kwargs={
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_edit_get_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_action_edit',
            kwargs={
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }
        )

    def _request_test_workflow_template_state_action_edit_post_view(
        self, extra_data=None
    ):
        data = {
            'label': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_LABEL_EDITED,
            'when': self._test_workflow_template_state_action.when
        }

        if extra_data:
            data.update(**extra_data)

        return self.post(
            viewname='document_states:workflow_template_state_action_edit',
            kwargs={
                'workflow_template_state_action_id': self._test_workflow_template_state_action.pk
            }, data=data
        )

    def _request_test_workflow_template_state_action_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }
        )

    def _request_test_workflow_template_state_action_selection_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_selection',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_state.pk
            }, data={
                'klass': TEST_WORKFLOW_TEMPLATE_STATE_ACTION_DOTTED_PATH
            }
        )



from django.conf import settings
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig

from .handlers import handler_auto_admin_account_password_change


class AutoAdminAppConfig(MayanAppConfig):
    has_tests = True
    name = 'mayan.apps.autoadmin'
    verbose_name = _('Auto administrator')

    def ready(self):
        super().ready()

        post_save.connect(
            dispatch_uid='autoadmin_handler_account_password_change',
            receiver=handler_auto_admin_account_password_change,
            sender=settings.AUTH_USER_MODEL
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.dynamic_search.search_models import SearchModel

from .permissions import permission_index_instance_view

search_model_index_instance_node = SearchModel(
    app_label='document_indexing', model_name='IndexInstanceNodeSearchResult',
    permission=permission_index_instance_view,
    serializer_path='mayan.apps.document_indexing.serializers.IndexInstanceNodeSerializer'
)
search_model_index_instance_node.add_proxy_model(
    app_label='document_indexing', model_name='IndexInstanceNode'
)

search_model_index_instance_node.add_model_field(
    field='value', label=_('Value')
)

search_model_index_instance_node.add_model_field(
    field='documents__document_type__label', label=_('Document type')
)
search_model_index_instance_node.add_model_field(
    field='documents__files__mimetype', label=_('Document MIME type')
)
search_model_index_instance_node.add_model_field(
    field='documents__label', label=_('Document label')
)
search_model_index_instance_node.add_model_field(
    field='documents__description', label=_('Document description')
)
search_model_index_instance_node.add_model_field(
    field='documents__uuid', label=_('Document UUID')
)
search_model_index_instance_node.add_model_field(
    field='documents__files__checksum', label=_('Document checksum')
)



from django.conf.urls import url

from .views import DashboardDetailView, DashboardListView

urlpatterns = [
    url(
        regex=r'^dashboards/$', name='dashboard_list',
        view=DashboardListView.as_view()
    ),
    url(
        regex=r'^dashboards/(?P<dashboard_name>[-\w]+)/$',
        name='dashboard_detail', view=DashboardDetailView.as_view()
    )
]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Sources'), name='sources'
)

event_source_created = namespace.add_event_type(
    label=_('Source created'), name='source_create'
)
event_source_edited = namespace.add_event_type(
    label=_('Source edited'), name='source_edit'
)



from mayan.apps.documents.events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_version_created,
    event_document_version_page_created
)
from mayan.apps.documents.document_file_actions import (
    DocumentFileActionAppendNewPages, DocumentFileActionNothing,
    DocumentFileActionUseNewPages
)

from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import (
    TEST_DOCUMENT_SMALL_CHECKSUM, TEST_FILE_COMPRESSED_PATH,
    TEST_FILE_SMALL_PATH
)
from mayan.apps.source_compressed.source_backends.literals import (
    SOURCE_UNCOMPRESS_CHOICE_ALWAYS, SOURCE_UNCOMPRESS_CHOICE_ASK,
    SOURCE_UNCOMPRESS_CHOICE_NEVER
)

from .mixins import WebFormSourceTestMixin


class WebFormSourceBackendActionDocumentFileUploadTestCase(
    WebFormSourceTestMixin, GenericDocumentTestCase
):
    _test_source_create_auto = False
    auto_upload_test_document = True

    def test_basic(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={'file_object': file_object}
            )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, test_document_version_page)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_basic_trashed_document(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._test_document.delete()

        self._clear_events()

        with self.assertRaises(expected_exception=Document.DoesNotExist):
            with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
                self._execute_test_source_action(
                    action_name='document_file_upload',
                    extra_data={'file_object': file_object}
                )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_comment(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'comment': 'test-comment',
                    'file_object': file_object
                }
            )

        self._test_document.refresh_from_db()

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.file_latest.comment, 'test-comment'
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, test_document_version_page)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_document_file_action_append(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'document_file_action_name': DocumentFileActionAppendNewPages.action_id,
                    'file_object': file_object
                }
            )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(),
            self._test_document.files.all()[0].pages.count() + self._test_document.files.all()[1].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page_first = test_document_version.pages.all()[0]
        test_document_version_page_last = test_document_version.pages.all()[1]

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, test_document_version_page_first)
        self.assertEqual(events[3].target, test_document_version_page_first)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page_last)
        self.assertEqual(events[4].target, test_document_version_page_last)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_document_file_action_keep(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'document_file_action_name': DocumentFileActionNothing.action_id,
                    'file_object': file_object
                }
            )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

    def test_document_file_action_new(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'document_file_action_name': DocumentFileActionUseNewPages.action_id,
                    'file_object': file_object
                }
            )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, test_document_version_page)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_filename(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'file_object': file_object,
                    'filename': 'test-filename'
                }
            )

        self._test_document.refresh_from_db()

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.file_latest.filename, 'test-filename'
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, test_document_file)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, test_document_version_page)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )

    def test_user(self):
        self._test_source_create()

        test_document_count = Document.objects.count()
        test_document_file_count = self._test_document.files.count()
        test_document_version_count = self._test_document.versions.count()
        test_document_version_page_count = self._test_document.versions.all()[0].pages.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_file_upload',
                extra_data={
                    'file_object': file_object,
                    'user': self._test_case_user
                }
            )

        self.assertEqual(Document.objects.count(), test_document_count)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )
        self.assertEqual(
            self._test_document.versions.count(),
            test_document_version_count + 1
        )
        self.assertEqual(
            self._test_document.versions.all()[0].pages.count(),
            test_document_version_page_count
        )
        self.assertEqual(
            self._test_document.versions.all()[1].pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version_page)
        self.assertEqual(
            events[3].verb, event_document_version_page_created.id
        )


class WebFormSourceBackendActionDocumentUploadTestCase(
    WebFormSourceTestMixin, GenericDocumentTestCase
):
    _test_source_create_auto = False
    auto_upload_test_document = False

    def test_basic(self):
        self._test_source_create()

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={'file_object': file_object}
            )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_compressed_always(self):
        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_ALWAYS}
        )

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_COMPRESSED_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={'file_object': file_object}
            )

        self.assertEqual(Document.objects.count(), document_count + 2)

        self.assertTrue(
            'first document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )
        self.assertTrue(
            'second document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 11)

        test_documents = Document.objects.all()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_documents[0])
        self.assertEqual(events[0].target, test_documents[0])
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_documents[0])
        self.assertEqual(events[1].actor, test_documents[0].file_latest)
        self.assertEqual(events[1].target, test_documents[0].file_latest)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_documents[0])
        self.assertEqual(events[2].actor, test_documents[0].file_latest)
        self.assertEqual(events[2].target, test_documents[0].file_latest)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_documents[0])
        self.assertEqual(events[3].actor, test_documents[0].version_active)
        self.assertEqual(events[3].target, test_documents[0].version_active)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(
            events[4].action_object, test_documents[0].version_active
        )
        self.assertEqual(
            events[4].actor, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].target, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, self._test_document_type)
        self.assertEqual(events[5].actor, test_documents[1])
        self.assertEqual(events[5].target, test_documents[1])
        self.assertEqual(events[5].verb, event_document_created.id)

        self.assertEqual(events[6].action_object, test_documents[1])
        self.assertEqual(events[6].actor, test_documents[1].file_latest)
        self.assertEqual(events[6].target, test_documents[1].file_latest)
        self.assertEqual(events[6].verb, event_document_file_created.id)

        self.assertEqual(events[7].action_object, test_documents[1])
        self.assertEqual(events[7].actor, test_documents[1].file_latest)
        self.assertEqual(events[7].target, test_documents[1].file_latest)
        self.assertEqual(events[7].verb, event_document_file_edited.id)

        self.assertEqual(events[8].action_object, test_documents[1])
        self.assertEqual(events[8].actor, test_documents[1].version_active)
        self.assertEqual(events[8].target, test_documents[1].version_active)
        self.assertEqual(events[8].verb, event_document_version_created.id)

        self.assertEqual(
            events[9].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[9].actor, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].target, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].verb, event_document_version_page_created.id
        )

        self.assertEqual(
            events[10].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[10].actor, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].target, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].verb, event_document_version_page_created.id
        )

    def test_compressed_ask_false(self):
        self._silence_logger(name='mayan.apps.converter.backends')

        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_ASK}
        )

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_COMPRESSED_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={
                    'expand': False,
                    'file_object': file_object
                }
            )

        self.assertEqual(Document.objects.count(), document_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

    def test_compressed_ask_true(self):
        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_ASK}
        )

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_COMPRESSED_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={
                    'expand': True,
                    'file_object': file_object
                }
            )

        self.assertEqual(Document.objects.count(), document_count + 2)

        self.assertTrue(
            'first document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )
        self.assertTrue(
            'second document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 11)

        test_documents = Document.objects.all()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_documents[0])
        self.assertEqual(events[0].target, test_documents[0])
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_documents[0])
        self.assertEqual(events[1].actor, test_documents[0].file_latest)
        self.assertEqual(events[1].target, test_documents[0].file_latest)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_documents[0])
        self.assertEqual(events[2].actor, test_documents[0].file_latest)
        self.assertEqual(events[2].target, test_documents[0].file_latest)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_documents[0])
        self.assertEqual(events[3].actor, test_documents[0].version_active)
        self.assertEqual(events[3].target, test_documents[0].version_active)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(
            events[4].action_object, test_documents[0].version_active
        )
        self.assertEqual(
            events[4].actor, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].target, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, self._test_document_type)
        self.assertEqual(events[5].actor, test_documents[1])
        self.assertEqual(events[5].target, test_documents[1])
        self.assertEqual(events[5].verb, event_document_created.id)

        self.assertEqual(events[6].action_object, test_documents[1])
        self.assertEqual(events[6].actor, test_documents[1].file_latest)
        self.assertEqual(events[6].target, test_documents[1].file_latest)
        self.assertEqual(events[6].verb, event_document_file_created.id)

        self.assertEqual(events[7].action_object, test_documents[1])
        self.assertEqual(events[7].actor, test_documents[1].file_latest)
        self.assertEqual(events[7].target, test_documents[1].file_latest)
        self.assertEqual(events[7].verb, event_document_file_edited.id)

        self.assertEqual(events[8].action_object, test_documents[1])
        self.assertEqual(events[8].actor, test_documents[1].version_active)
        self.assertEqual(events[8].target, test_documents[1].version_active)
        self.assertEqual(events[8].verb, event_document_version_created.id)

        self.assertEqual(
            events[9].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[9].actor, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].target, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].verb, event_document_version_page_created.id
        )

        self.assertEqual(
            events[10].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[10].actor, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].target, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].verb, event_document_version_page_created.id
        )

    def test_compressed_never(self):
        self._silence_logger(name='mayan.apps.converter.backends')

        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_NEVER}
        )

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_COMPRESSED_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={'file_object': file_object}
            )

        self.assertEqual(Document.objects.count(), document_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

    def test_label(self):
        self._test_source_create()

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={
                    'file_object': file_object,
                    'label': 'test-label'
                }
            )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )
        self.assertEqual(
            Document.objects.first().label, 'test-label'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_language(self):
        self._test_source_create()

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={
                    'file_object': file_object,
                    'language': 'deu'
                }
            )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )
        self.assertEqual(
            Document.objects.first().language, 'deu'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_user(self):
        self._test_source_create()

        document_count = Document.objects.count()

        self._clear_events()

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_object:
            self._execute_test_source_action(
                action_name='document_upload',
                extra_data={
                    'file_object': file_object,
                    'user': self._test_case_user
                }
            )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, self._test_case_user)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_credential_created, event_credential_edited
from ..models import StoredCredential
from ..permissions import (
    permission_credential_create, permission_credential_delete,
    permission_credential_edit, permission_credential_view
)

from .mixins import StoredCredentialViewTestMixin


class CredentialViewTestCase(
    StoredCredentialViewTestMixin, GenericViewTestCase
):
    auto_create_test_credential = False

    def setUp(self):
        super().setUp()
        # Hidden immport register the test credential backend and
        # allow tests to access it.
        from .credential_backends import TestCredentialBackend  # NOQA

    def test_stored_credential_backend_selection_view_no_permissions(self):
        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_backend_selection_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(StoredCredential.objects.count(), credential_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_backend_selection_view_with_permissions(self):
        self.grant_permission(permission=permission_credential_create)

        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_backend_selection_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(StoredCredential.objects.count(), credential_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_create_view_no_permissions(self):
        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(StoredCredential.objects.count(), credential_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_create_view_with_permissions(self):
        self.grant_permission(permission=permission_credential_create)

        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            StoredCredential.objects.count(), credential_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_stored_credential)
        self.assertEqual(events[0].verb, event_credential_created.id)

    def test_stored_credential_delete_view_no_permissions(self):
        self._create_test_stored_credential()

        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(StoredCredential.objects.count(), credential_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_delete_view_with_access(self):
        self._create_test_stored_credential()

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_delete
        )

        credential_count = StoredCredential.objects.count()

        self._clear_events()

        response = self._request_test_stored_credential_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            StoredCredential.objects.count(), credential_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_edit_view_no_permissions(self):
        self._create_test_stored_credential()

        credential_label = self._test_stored_credential.label

        self._clear_events()

        response = self._request_test_stored_credential_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_stored_credential.refresh_from_db()
        self.assertEqual(
            self._test_stored_credential.label, credential_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_edit_view_with_access(self):
        self._create_test_stored_credential()

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_edit
        )

        credential_label = self._test_stored_credential.label

        self._clear_events()

        response = self._request_test_stored_credential_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_stored_credential.refresh_from_db()
        self.assertNotEqual(
            self._test_stored_credential.label, credential_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object, None
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_stored_credential)
        self.assertEqual(events[0].verb, event_credential_edited.id)

    def test_stored_credential_list_view_with_no_permission(self):
        self._create_test_stored_credential()

        self._clear_events()

        response = self._request_test_stored_credential_list_view()
        self.assertNotContains(
            response=response, text=self._test_stored_credential.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_stored_credential_list_view_with_access(self):
        self._create_test_stored_credential()

        self.grant_access(
            obj=self._test_stored_credential,
            permission=permission_credential_view
        )

        self._clear_events()

        response = self._request_test_stored_credential_list_view()
        self.assertContains(
            response=response, text=self._test_stored_credential.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel

from mayan.apps.documents.models.document_models import Document

from ..managers import (
    DocumentIndexInstanceNodeManager, IndexInstanceManager
)

from .index_template_models import IndexTemplate, IndexTemplateNode
from .index_instance_model_mixins import (
    IndexInstanceBusinessLogicMixin, IndexInstanceNodeBusinessLogicMixin
)


class IndexInstance(IndexInstanceBusinessLogicMixin, IndexTemplate):
    """
    Model that represents an evaluated index. This is an index whose nodes
    have been evaluated against a series of documents. If is a proxy model
    at the moment.
    """
    objects = IndexInstanceManager()

    class Meta:
        proxy = True
        verbose_name = _('Index instance')
        verbose_name_plural = _('Index instances')

    def get_absolute_url(self):
        try:
            index_instance_root_node = self.index_instance_root_node
        except IndexInstanceNode.DoesNotExist:
            return '#'
        else:
            return reverse(
                viewname='indexing:index_instance_node_view', kwargs={
                    'index_instance_node_id': index_instance_root_node.pk
                }
            )


class IndexInstanceNode(IndexInstanceNodeBusinessLogicMixin, MPTTModel):
    """
    This model represent one instance node from a index template node. That is
    a node template that has been evaluated against a document and the result
    from that evaluation is this node's stored values. Instances of this
    model also point to the original node template.
    """
    parent = TreeForeignKey(
        blank=True, null=True, on_delete=models.CASCADE,
        related_name='children', to='self'
    )
    index_template_node = models.ForeignKey(
        on_delete=models.CASCADE, related_name='index_instance_nodes',
        to=IndexTemplateNode, verbose_name=_('Index template node')
    )
    value = models.CharField(
        blank=True, db_index=True, max_length=255, verbose_name=_('Value')
    )
    documents = models.ManyToManyField(
        related_name='index_instance_nodes', to=Document,
        verbose_name=_('Documents')
    )

    class Meta:
        ordering = ('value',)
        unique_together = ('index_template_node', 'parent', 'value')
        verbose_name = _('Index instance node')
        verbose_name_plural = _('Indexes instances node')

    def __str__(self):
        return self.value

    def get_absolute_url(self):
        return reverse(
            viewname='indexing:index_instance_node_view', kwargs={
                'index_instance_node_id': self.pk
            }
        )


class DocumentIndexInstanceNode(IndexInstanceNode):
    """
    Proxy model of node instance. It is used to represent the node instance
    in which a document is currently located. It is used to aid in column
    registration. The inherited methods of this model should not be used.
    """
    objects = DocumentIndexInstanceNodeManager()

    class Meta:
        proxy = True
        verbose_name = _('Document index node instance')
        verbose_name_plural = _('Document indexes node instances')


class IndexInstanceNodeSearchResult(IndexInstanceNode):
    class Meta:
        proxy = True
        verbose_name = _('Index instance node')
        verbose_name_plural = _('Index instance nodes')



from django.db import migrations
from django.db.models import Count


def code_remove_duplicates(apps, schema_editor):
    WorkflowTransitionTriggerEvent = apps.get_model(
        app_label='document_states',
        model_name='WorkflowTransitionTriggerEvent'
    )

    queryset = WorkflowTransitionTriggerEvent.objects.values(
        'transition', 'event_type'
    ).annotate(count=Count('id')).order_by().filter(count__gt=1)

    for workflow_transition_trigger_entry in queryset:
        workflow_transition_trigger_entry.pop('count')

        queryset_to_delete = WorkflowTransitionTriggerEvent.objects.filter(
            **workflow_transition_trigger_entry
        )[1:].values('id')

        # Redo query as 'limit' or 'offset' can't be used with delete.
        WorkflowTransitionTriggerEvent.objects.filter(
            id__in=queryset_to_delete
        ).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0023_auto_20200930_0726')
    ]

    operations = [
        migrations.RunPython(
            code=code_remove_duplicates,
            reverse_code=migrations.RunPython.noop
        )
    ]



from mayan.apps.testing.tests.base import BaseTestCase

from .literals import (
    MAILER_BACKEND_DJANGO_FILE_PATH, MAILER_BACKEND_DJANGO_SMTP_PATH,
    TEST_DJANGO_FILE_PATH
)
from .mixins import MailerTestMixin


class DjangoMailerBackendTestCase(MailerTestMixin, BaseTestCase):
    _test_mailer_backend_path = MAILER_BACKEND_DJANGO_SMTP_PATH

    def test_mailer_connection_kwargs(self):
        self._create_test_user_mailer(
            extra_backend_data={
                'host': '127.0.0.1',
                'port': '999',
                'stored_credential_id': self._test_stored_credential.pk
            }
        )

        backend_instance = self._test_user_mailer.get_backend_instance()
        connection_kwargs = backend_instance.get_connection_kwargs()

        self.assertEqual(
            connection_kwargs['password'],
            self._test_stored_credential_backend_data['password']
        )
        self.assertEqual(
            connection_kwargs['username'],
            self._test_stored_credential_backend_data['username']
        )


class DjangoFileMailerTestCase(MailerTestMixin, BaseTestCase):
    _test_mailer_backend_path = MAILER_BACKEND_DJANGO_FILE_PATH

    def test_mailer_connection_kwargs(self):
        self._create_test_user_mailer(
            extra_backend_data={'file_path': TEST_DJANGO_FILE_PATH}
        )

        backend_instance = self._test_user_mailer.get_backend_instance()
        connection_kwargs = backend_instance.get_connection_kwargs()

        self.assertEqual(
            connection_kwargs['file_path'], TEST_DJANGO_FILE_PATH
        )



class StatisticsViewTestMixin:
    def _request_test_statistic_detail_view(self):
        return self.get(
            viewname='statistics:statistic_detail', kwargs={
                'slug': self.statistic.slug
            }
        )

    def _request_test_namespace_list_view(self):
        return self.get(viewname='statistics:statistic_namespace_list')



class OwnerPlusFilteredQuerysetAPIViewMixin:
    def filter_queryset(self, queryset):
        queryset_user_owned = queryset.filter(user=self.request.user)

        queryset_filtered = super().filter_queryset(queryset=queryset)

        queryset = queryset_user_owned | queryset_filtered
        return queryset.distinct()



from django.utils.translation import ugettext_lazy as _

FIELDS_ALL = ('username', 'first_name', 'last_name', 'email', 'is_active')
FIELDS_USER = ('first_name', 'last_name')

FIELDSETS_ALL = (
    (
        _('Account'), {
            'fields': ('username', 'email'),
        }
    ), (
        _('Personal'), {
            'fields': ('first_name', 'last_name')
        },
    ), (
        _('Attributes'), {
            'fields': ('is_active',)
        },
    )
)
FIELDSETS_USER = (
    (
        _('Personal'), {
            'fields': ('first_name', 'last_name')
        },
    ),
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Checkouts'), name='checkouts'
)

event_document_auto_checked_in = namespace.add_event_type(
    label=_('Document automatically checked in'),
    name='document_auto_check_in'
)
event_document_checked_in = namespace.add_event_type(
    label=_('Document checked in'), name='document_check_in'
)
event_document_checked_out = namespace.add_event_type(
    label=_('Document checked out'), name='document_check_out'
)
event_document_forcefully_checked_in = namespace.add_event_type(
    label=_('Document forcefully checked in'),
    name='document_forceful_check_in'
)



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0008_auto_20170803_0752'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkflowStateAction',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'label', models.CharField(
                        max_length=255, verbose_name='Label'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, verbose_name='Enabled'
                    )
                ),
                (
                    'when', models.PositiveIntegerField(
                        choices=[(1, 'On entry'), (2, 'On exit')], default=1,
                        help_text='At which moment of the state this action '
                        'will execute', verbose_name='When'
                    )
                ),
                (
                    'action_path', models.CharField(
                        help_text='The dotted Python path to the workflow '
                        'action class to execute.', max_length=128,
                        verbose_name='Entry action path'
                    )
                ),
                (
                    'action_data', models.TextField(
                        blank=True, verbose_name='Entry action data'
                    )
                ),
                (
                    'state', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='actions',
                        to='document_states.WorkflowState',
                        verbose_name='Workflow state'
                    )
                ),
            ],
            options={
                'ordering': ('label',),
                'verbose_name': 'Workflow state action',
                'verbose_name_plural': 'Workflow state actions',
            },
        ),
        migrations.AlterUniqueTogether(
            name='workflowstateaction',
            unique_together={('state', 'label')},
        ),
    ]



from django.apps import apps
from django.db.models import F, Max
from django.utils.translation import ugettext_lazy as _

from .classes import DuplicateBackend


class DuplicateBackendFileChecksum(DuplicateBackend):
    label = _('Exact document file checksum')

    @classmethod
    def verify(cls, document):
        return document.file_latest

    def process(self, document):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        # Get the documents whose latest file matches the checksum
        # of the current document and exclude the current document

        return Document.objects.annotate(
            max_timestamp=Max('files__timestamp')
        ).filter(
            files__timestamp=F('max_timestamp'),
            files__checksum=document.file_latest.checksum
        ).exclude(pk=document.pk)


class DuplicateBackendLabel(DuplicateBackend):
    label = _('Exact document label')

    def process(self, document):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        return Document.objects.filter(
            label=document.label
        ).exclude(pk=document.pk)



from django.db import migrations

from ..compatibility import RGBColorField


class Migration(migrations.Migration):
    dependencies = [
        ('tags', '0004_auto_20150717_2336'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='color',
            field=RGBColorField(verbose_name='Color'),
            preserve_default=True,
        ),
    ]



from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import (
    TEST_FILE_HYBRID_PDF_CONTENT, TEST_FILE_HYBRID_PDF_PATH,
    TEST_FILE_OFFICE_CONTENT, TEST_FILE_OFFICE_PATH,
    TEST_FILE_TEXT_CONTENT, TEST_FILE_TEXT_PATH
)

from ..parsers import OfficePopplerParser, PopplerParser


class ParserTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    def test_office_file_parser_with_office_doc_file(self):
        self._test_document_path = TEST_FILE_OFFICE_PATH
        self._upload_test_document()

        parser = OfficePopplerParser()

        parser.process_document_file(self._test_document_file)

        self.assertTrue(
            TEST_FILE_OFFICE_CONTENT in self._test_document_file.pages.first().content.content
        )

    def test_office_file_parser_with_text_file(self):
        self._test_document_path = TEST_FILE_TEXT_PATH
        self._upload_test_document()

        parser = OfficePopplerParser()

        parser.process_document_file(self._test_document_file)

        self.assertTrue(
            TEST_FILE_TEXT_CONTENT in self._test_document_file.pages.first().content.content
        )

    def test_poppler_parser_with_pdf(self):
        self._test_document_path = TEST_FILE_HYBRID_PDF_PATH
        self._upload_test_document()

        parser = PopplerParser()

        parser.process_document_file(self._test_document_file)

        self.assertTrue(
            TEST_FILE_HYBRID_PDF_CONTENT in self._test_document_file.pages.first().content.content
        )



from django.test import override_settings

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..classes import AJAXTemplate

from .literals import TEST_AJAXTEMPLATE_RESULT


class AJAXTemplateAPIViewTestCase(BaseAPITestCase):
    auto_login_user = False

    def test_template_detail_anonymous_api_view(self):
        template_main_menu = AJAXTemplate.get(name='menu_main')

        response = self.get(path=template_main_menu.get_absolute_url())
        self.assertNotContains(
            response=response, text=TEST_AJAXTEMPLATE_RESULT, status_code=403
        )

    @override_settings(LANGUAGE_CODE='de')
    def test_template_detail_api_view(self):
        self.login_user()
        template_main_menu = AJAXTemplate.get(name='menu_main')

        response = self.get(path=template_main_menu.get_absolute_url())
        self.assertContains(
            response=response, text=TEST_AJAXTEMPLATE_RESULT, status_code=200
        )



from mayan.apps.rest_api import generics
from mayan.apps.views.generics import DownloadViewMixin

from .api_view_mixins import OwnerPlusFilteredQuerysetAPIViewMixin
from .models import DownloadFile
from .permissions import (
    permission_download_file_delete, permission_download_file_download,
    permission_download_file_view
)
from .serializers import DownloadFileSerializer


class APIDownloadFileDetailView(
    OwnerPlusFilteredQuerysetAPIViewMixin, generics.RetrieveDestroyAPIView
):
    """
    delete: Delete the selected download file.
    get: Return the details of the selected download file.
    """
    lookup_url_kwarg = 'download_file_id'
    mayan_object_permissions = {
        'DELETE': (permission_download_file_delete,),
        'GET': (permission_download_file_view,),
    }
    serializer_class = DownloadFileSerializer
    source_queryset = DownloadFile.objects.all()

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIDownloadFileDownloadView(
    DownloadViewMixin, OwnerPlusFilteredQuerysetAPIViewMixin,
    generics.RetrieveAPIView
):
    """
    get: Download a download file.
    """
    lookup_url_kwarg = 'download_file_id'
    mayan_object_permissions = {
        'GET': (permission_download_file_download,),
    }
    model = DownloadFile

    def get_download_file_object(self):
        instance = self.get_object()
        instance._event_actor = self.request.user
        return instance.get_download_file_object()

    def get_download_filename(self):
        return self.get_object().filename

    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def retrieve(self, request, *args, **kwargs):
        return self.render_to_response()


class APIDownloadFileListView(
    OwnerPlusFilteredQuerysetAPIViewMixin, generics.ListAPIView
):
    """
    get: Returns a list of all the download files.
    """
    mayan_object_permissions = {
        'GET': (permission_download_file_view,)
    }
    ordering_fields = ('filename', 'datetime', 'id', 'label')
    serializer_class = DownloadFileSerializer
    source_queryset = DownloadFile.objects.all()

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }



from django.utils.encoding import force_bytes

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..events import (
    event_ocr_document_version_content_deleted,
    event_ocr_document_version_page_content_edited,
    event_ocr_document_version_submitted, event_ocr_document_version_finished
)
from ..models import DocumentVersionPageOCRContent
from ..permissions import (
    permission_document_version_ocr_content_edit,
    permission_document_version_ocr_content_view,
    permission_document_version_ocr, permission_document_type_ocr_setup
)

from .literals import TEST_DOCUMENT_VERSION_OCR_CONTENT
from .mixins import (
    DocumentVersionOCRTestMixin, DocumentVersionOCRViewTestMixin,
    DocumentVersionPageOCRViewTestMixin, DocumentTypeOCRViewTestMixin
)


class DocumentTypeOCRViewTestCase(
    DocumentTypeOCRViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_document_type_ocr_settings_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_type_ocr_settings_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_settings_view_with_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_ocr_setup
        )

        self._clear_events()

        response = self._request_test_document_type_ocr_settings_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_submit_view_no_permission(self):
        self._upload_test_document()

        self._clear_events()

        response = self._request_document_type_ocr_submit_view()
        self.assertEqual(response.status_code, 200)

        self.assertFalse(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_ocr_submit_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_document_type_ocr_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_submitted.id
        )

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_ocr_document_version_finished.id
        )

    def test_trashed_document_type_ocr_submit_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_document_type_ocr_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentVersionOCRViewTestCase(
    DocumentVersionOCRViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_verions_ocr_content_single_delete_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_single_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_content_single_delete_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_content_single_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_content_deleted.id
        )

    def test_trashed_document_version_ocr_content_single_delete_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_single_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_content_multiple_delete_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_multiple_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_content_multiple_delete_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_content_multiple_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_content_deleted.id
        )

    def test_trashed_document_version_ocr_content_multiple_delete_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document, permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_multiple_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            DocumentVersionPageOCRContent.objects.filter(
                document_version_page=self._test_document_version.pages.first()
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_content_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_content_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_content_detail_view()
        self.assertContains(
            response=response, text=TEST_DOCUMENT_VERSION_OCR_CONTENT,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_ocr_content_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_submit_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_ocr_single_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            ''.join(self._test_document_version.ocr_content()), ''
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_submit_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_single_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_submitted.id
        )

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_ocr_document_version_finished.id
        )

    def test_trashed_document_version_ocr_submit_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_single_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertFalse(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_multiple_submit_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_ocr_multiple_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            ''.join(self._test_document_version.ocr_content()), ''
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_multiple_submit_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_multiple_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_submitted.id
        )

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_ocr_document_version_finished.id
        )

    def test_trashed_document_version_ocr_multiple_submit_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_multiple_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertFalse(
            TEST_DOCUMENT_VERSION_OCR_CONTENT in ''.join(
                self._test_document_version.ocr_content()
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_download_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_download_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_ocr_download_view_with_access(self):
        self.expected_content_types = ('application/octet-stream',)

        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._clear_events()

        response = self._request_test_document_version_ocr_content_download_view()
        self.assertEqual(response.status_code, 200)

        self.assert_download_response(
            response=response, content=force_bytes(
                s=''.join(
                    self._test_document.ocr_content()
                )
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_ocr_download_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_ocr_content_download_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentVersionPageOCRViewTestCase(
    DocumentVersionOCRTestMixin, DocumentVersionPageOCRViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_version_page_ocr_content_detail_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_page_ocr_content_detail_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_detail_view()
        self.assertContains(
            response=response, text=TEST_DOCUMENT_VERSION_OCR_CONTENT,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_page_ocr_content_detail_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_page_ocr_content_edit_view_no_permission(self):
        self._create_test_document_version_ocr_content()

        test_document_version_page_ocr_content = self._test_document_version_page.ocr_content.content
        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version_page.ocr_content.refresh_from_db()
        self.assertEqual(
            self._test_document_version_page.ocr_content.content,
            test_document_version_page_ocr_content
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_page_ocr_content_edit_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_edit
        )

        test_document_version_page_ocr_content = self._test_document_version_page.ocr_content.content
        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_version_page.ocr_content.refresh_from_db()
        self.assertNotEqual(
            self._test_document_version_page.ocr_content.content,
            test_document_version_page_ocr_content
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version_page)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_page_content_edited.id
        )

    def test_trashed_document_version_page_ocr_content_edit_view_with_access(self):
        self._create_test_document_version_ocr_content()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr_content_edit
        )

        test_document_version_page_ocr_content = self._test_document_version_page.ocr_content.content
        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_page_ocr_content_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version_page.ocr_content.refresh_from_db()
        self.assertEqual(
            self._test_document_version_page.ocr_content.content,
            test_document_version_page_ocr_content
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.forms import FilteredSelectionForm
from mayan.apps.views.widgets import ColorWidget

from .models import Tag
from .widgets import TagFormWidget


class TagForm(forms.ModelForm):
    class Meta:
        fields = ('label', 'color')
        model = Tag
        widgets = {
            'color': ColorWidget()
        }


class TagMultipleSelectionForm(FilteredSelectionForm):
    class Media:
        js = ('tags/js/tags_form.js',)

    class Meta:
        allow_multiple = True
        field_name = 'tags'
        label = _('Tags')
        required = False
        widget_class = TagFormWidget
        widget_attributes = {'class': 'select2-tags'}



import logging

from django.apps import apps

from .events import event_file_metadata_document_file_finished
from .exceptions import FileMetadataDriverError

logger = logging.getLogger(name=__name__)


class FileMetadataDriver:
    _registry = {}

    @classmethod
    def process_document_file(cls, document_file, user=None):
        # Get list of drivers for the document's MIME type
        driver_classes = cls._registry.get(document_file.mimetype, ())
        # Add wilcard drivers, drivers meant to be executed for all MIME
        # types.
        driver_classes += tuple(cls._registry.get('*', ()))

        for driver_class in driver_classes:
            try:
                driver = driver_class()

                driver.initialize()

                driver.process(document_file=document_file)

                event_file_metadata_document_file_finished.commit(
                    action_object=document_file.document, actor=user,
                    target=document_file
                )

            except FileMetadataDriverError:
                """If driver raises error, try next in the list."""
            else:
                # If driver was successful there is no need to try
                # others in the list for this mimetype.
                return

    @classmethod
    def register(cls, mimetypes):
        for mimetype in mimetypes:
            cls._registry.setdefault(mimetype, []).append(cls)

    def __init__(self, auto_initialize=True, **kwargs):
        self.auto_initialize = auto_initialize

    def get_driver_path(self):
        return '.'.join(
            [self.__module__, self.__class__.__name__]
        )

    def initialize(self):
        StoredDriver = apps.get_model(
            app_label='file_metadata', model_name='StoredDriver'
        )

        driver_path = self.get_driver_path()

        self.driver_model, created = StoredDriver.objects.get_or_create(
            driver_path=driver_path, defaults={
                'internal_name': self.internal_name
            }
        )

    def process(self, document_file):
        logger.info(
            'Starting processing document file: %s', document_file
        )

        self.driver_model.driver_entries.filter(
            document_file=document_file
        ).delete()

        document_file_driver_entry = self.driver_model.driver_entries.create(
            document_file=document_file
        )

        results = self._process(document_file=document_file) or {}

        for key, value in results.items():
            document_file_driver_entry.entries.create(
                key=key, value=value
            )

    def _process(self, document_file):
        raise NotImplementedError(
            'Your %s class has not defined the required '
            '_process() method.' % self.__class__.__name__
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.authentication.link_conditions import (
    condition_user_is_authenticated
)
from mayan.apps.navigation.classes import Link

from .icons import (
    icon_user_locale_profile_detail, icon_user_locale_profile_edit
)


link_user_locale_profile_detail = Link(
    args='object.id', condition=condition_user_is_authenticated,
    icon=icon_user_locale_profile_detail, text=_('Locale profile'),
    view='locales:user_locale_profile_detail'
)
link_user_locale_profile_edit = Link(
    args='object.id', condition=condition_user_is_authenticated,
    icon=icon_user_locale_profile_edit, text=_('Edit locale profile'),
    view='locales:user_locale_profile_edit'
)



from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.document_indexing.models.index_instance_models import IndexInstanceNode
from mayan.apps.document_indexing.tests.mixins import IndexTemplateTestMixin

from .literals import TEST_CABINET_LABEL_EDITED, TEST_INDEX_NODE_TEMPLATE
from .mixins import CabinetTestMixin


class CabinetIndexingTestCase(
    CabinetTestMixin, IndexTemplateTestMixin, GenericDocumentTestCase
):
    _test_index_template_node_expression = TEST_INDEX_NODE_TEMPLATE
    auto_create_test_cabinet = True
    auto_create_test_document_stub = True
    auto_upload_test_document = False

    def test_indexing_cabinet_add(self):
        self._test_cabinet.document_add(
            document=self._test_document, user=self._test_case_user
        )

        self.assertTrue(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=self._test_cabinet.label
            ).exists()
        )

    def test_indexing_cabinet_delete(self):
        self._test_cabinet.document_add(
            document=self._test_document, user=self._test_case_user
        )
        self._test_cabinet.delete()

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=self._test_cabinet.label
            ).exists()
        )

    def test_indexing_cabinet_edit(self):
        test_cabinet_label = self._test_cabinet.label

        self._test_cabinet.document_add(
            document=self._test_document, user=self._test_case_user
        )
        self._test_cabinet.label = TEST_CABINET_LABEL_EDITED
        self._test_cabinet.save()

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=test_cabinet_label
            ).exists()
        )
        self.assertTrue(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=TEST_CABINET_LABEL_EDITED
            ).exists()
        )

    def test_indexing_cabinet_remove(self):
        test_cabinet_label = self._test_cabinet.label

        self._test_cabinet.document_add(
            document=self._test_document, user=self._test_case_user
        )
        self._test_cabinet.document_remove(
            document=self._test_document, user=self._test_case_user
        )

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=test_cabinet_label
            ).exists()
        )



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0057_auto_20200916_1057'),
        ('file_metadata', '0003_auto_20191226_0606')
    ]

    operations = [
        migrations.RenameField(
            model_name='documentversiondriverentry',
            old_name='document_version',
            new_name='document_file'
        )
    ]



from .document_file_tasks import *  # NOQA
from .document_tasks import *  # NOQA
from .document_type_tasks import *  # NOQA
from .document_version_tasks import *  # NOQA
from .trashed_document_tasks import *  # NOQA



from django.utils.translation import ugettext_lazy as _

DEFAULT_EVENT_LIST_EXPORT_FILENAME = 'events_list.csv'

DEFAULT_EVENTS_DISABLE_ASYNCHRONOUS_MODE = False

EVENT_MANAGER_ORDER_AFTER = 1
EVENT_MANAGER_ORDER_BEFORE = 2

EVENT_TYPE_NAMESPACE_NAME = 'events'
EVENT_EVENTS_CLEARED_NAME = 'event_cleared'
EVENT_EVENTS_EXPORTED_NAME = 'event_exported'

TEXT_UNKNOWN_EVENT_ID = _('Unknown or obsolete event type: %s')



from django.conf.urls import url
from django.contrib import admin

from django.views.i18n import JavaScriptCatalog

from .api_views import APIContentTypeDetailView, APIContentTypeListView
from .views import (
    AboutView, FaviconRedirectView, HomeView, LicenseView, ObjectCopyView,
    RootView, SetupListView, ToolsListView
)

urlpatterns_misc = [
    url(
        regex=r'^favicon\.ico$', view=FaviconRedirectView.as_view()
    ),
    url(
        regex=r'^jsi18n/(?P<packages>\S+?)/$', name='javascript_catalog',
        view=JavaScriptCatalog.as_view()
    ),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/copy/$',
        name='object_copy', view=ObjectCopyView.as_view()
    )
]

urlpatterns = [
    url(
        regex=r'^$', name='root', view=RootView.as_view()
    ),
    url(
        regex=r'^home/$', name='home', view=HomeView.as_view()
    ),
    url(
        regex=r'^about/$', name='about_view', view=AboutView.as_view()
    ),
    url(
        regex=r'^license/$', name='license_view', view=LicenseView.as_view()
    ),
    url(
        regex=r'^setup/$', name='setup_list', view=SetupListView.as_view()
    ),
    url(
        regex=r'^tools/$', name='tools_list', view=ToolsListView.as_view()
    )
]

urlpatterns.extend(urlpatterns_misc)

passthru_urlpatterns = [
    url(regex=r'^admin/', view=admin.site.urls)
]

api_urls = [
    url(
        regex=r'^content_types/$', view=APIContentTypeListView.as_view(),
        name='content_type-list'
    ),
    url(
        regex=r'^content_types/(?P<content_type_id>[0-9]+)/$',
        name='content_type-detail', view=APIContentTypeDetailView.as_view()
    )
]



from django.db import models

from .events import event_error_log_deleted


class ErrorLogPartitionEntryManager(models.Manager):
    def clear(self, user=None):
        content_object = self.instance.content_object
        self.all().delete()

        event_error_log_deleted.commit(actor=user, target=content_object)



class FormOptions:
    def __init__(self, form, kwargs, options=None):
        """
        Option definitions will be iterated. The option value will be
        determined in the following order: as passed via keyword
        arguments during form intialization, as form get_... method or
        finally as static Meta options. This is to allow a form with
        Meta options or method to be overridden at initialization
        and increase the usability of a single class.
        """
        for name, default_value in self.option_definitions.items():
            try:
                # Check for a runtime value via kwargs
                value = kwargs.pop(name)
            except KeyError:
                try:
                    # Check if there is a get_... method
                    value = getattr(
                        self, 'get_{}'.format(name)
                    )()
                except AttributeError:
                    try:
                        # Check the meta class options
                        value = getattr(options, name)
                    except AttributeError:
                        value = default_value

            setattr(self, name, value)


class DetailFormOption(FormOptions):
    # Dictionary list of option names and default values.
    option_definitions = {
        'extra_fields': []
    }


class FilteredSelectionFormOptions(FormOptions):
    # Dictionary list of option names and default values.
    option_definitions = {
        'allow_multiple': False,
        'field_name': None,
        'help_text': None,
        'label': None,
        'model': None,
        'permission': None,
        'queryset': None,
        'required': True,
        'user': None,
        'widget_attributes': {'size': '10'},
        'widget_class': None
    }
